html, body, div, span, applet, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, big, cite, code, del, dfn, em, img, ins, kbd, q, s, samp, small, strike, strong, sub, sup, tt, var, b, u, i, center, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td, article, aside, canvas, details, figcaption, figure, footer, header, hgroup, menu, nav, section, summary, time, mark, audio, video {

    margin: 0;
    padding: 0;
    border: 0;
    outline: 0;
    font-size: 100%;
    font: inherit;
    vertical-align: baseline;

}
/* HTML5 display-role reset for older browsers */
article, aside, details, figcaption, figure, footer, header, hgroup, menu, nav, section {

    display: block;

}

/* Let's go for the actual style */

body {

	background-color: #EEEEEE;
	font-family: 'Noto Sans';

}

section {

	background-color: #FFFFFF;
	width: 90%;
	margin: 30px auto;
	padding: 10px 15px;

	box-shadow: 0px 1px 2px rgba(0,0,0, 0.25);

}

section > h2 {

	font-size: 200%;
	font-weight: bold;
	text-align: center;

}

h1.pagetitle {

	font-size: 300%;
	font-weight: bold;

	text-align: center;
	color: #2196F3;

}

h1.pagetitle > a {
	color: #2196F3;
}

a.backlink, a.backlink:link, a.backlink:visited, a.itemtitle, a.itemtitle:link, a.itemtitle:visited {

	color: #2196F3;

}

.buttons {

	text-align: center;

}

section > div.content, section > div.attachments {

	padding: 10px;

}

section > div.attachments > li.enclosure {

	list-style-type: circle;
	list-style-position: inside;

}

section > time, section > p.author {

	color: #888;
	font-size: 80%;
	padding: 10px;

}

button.backbutton, button.rss-feed {

	line-height: 1em;
	color: #FFF;
	font-weight: bold;
	vertical-align: middle;
	padding: 6px 12px;
	margin: 12px auto 0px;
	box-shadow: 0px 1px 2px rgba(0, 0, 0, 0.3);
	border-radius: 2px;
	border: 1px solid transparent;
	width: 200px;
	background: #2196F3 none repeat scroll 0% 0%;
	cursor: pointer;

	margin: 10px;


}

